package vn.hongyegroup.yybusiness.adapter;

import android.animation.LayoutTransition;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.recyclerview.widget.RecyclerView;

import com.android.basiclib.engine.image_load.ImageLoadExtKt;
import com.android.basiclib.utils.CheckUtil;
import com.android.basiclib.utils.CommUtils;

import me.yokeyword.indexablerv.IndexableAdapter;
import vn.hongyegroup.yybusiness.R;
import vn.hongyegroup.yybusiness.entity.AttendanceBean;

public class AttendanceAdapter extends IndexableAdapter<AttendanceBean> {

    private LayoutInflater mInflater;
    private LayoutTransition layoutTransition;

    public AttendanceAdapter(Context context) {
        mInflater = LayoutInflater.from(context);
        layoutTransition = new LayoutTransition();
    }

    @Override
    public RecyclerView.ViewHolder onCreateTitleViewHolder(ViewGroup parent) {
        View view = mInflater.inflate(R.layout.item_index_contact, parent, false);
        return new IndexVH(view);
    }

    @Override
    public void onBindTitleViewHolder(RecyclerView.ViewHolder holder, String indexTitle) {
        IndexVH vh = (IndexVH) holder;
        vh.tv.setText(indexTitle);
    }


    @Override
    public RecyclerView.ViewHolder onCreateContentViewHolder(ViewGroup parent) {
        View view = mInflater.inflate(R.layout.item_contact, parent, false);
        return new ContentVH(view);
    }

    @Override
    public void onBindContentViewHolder(RecyclerView.ViewHolder holder, AttendanceBean entity) {
        ContentVH vh = (ContentVH) holder;
        vh.tvName.setText(entity.staff_name);
        vh.tv_job_date.setText(entity.job_date);
        vh.tv_start_time.setText(entity.start_time);
        vh.tv_end_time.setText(entity.end_time);

        // 设置 LayoutTransition
        vh.ll_more_box.setLayoutTransition(layoutTransition);

        if (!CheckUtil.isEmpty(entity.check_in_time) && !CheckUtil.isEmpty(entity.check_in_img)) {
            vh.iv_in.setVisibility(View.VISIBLE);
            vh.tv_in.setVisibility(View.GONE);
            vh.tv_in_time.setVisibility(View.VISIBLE);
            vh.tv_in_time.setText(entity.check_in_time);
            ImageLoadExtKt.loadImage(vh.iv_in, entity.check_in_img,
                    0, 0, false,
                    false, 0, null,
                    false, false);
            vh.fl_sign_in_box.setBackground(CommUtils.getDrawable(R.drawable.shape_sign_white_bg));
        } else {
            vh.iv_in.setVisibility(View.GONE);
            vh.tv_in.setVisibility(View.VISIBLE);
            vh.tv_in_time.setVisibility(View.GONE);
            vh.fl_sign_in_box.setBackground(CommUtils.getDrawable(R.drawable.shape_sign_blue_bg));
        }

        if (!CheckUtil.isEmpty(entity.check_out_time) && !CheckUtil.isEmpty(entity.check_out_time)) {
            vh.iv_out.setVisibility(View.VISIBLE);
            vh.tv_out.setVisibility(View.GONE);
            vh.tv_out_time.setVisibility(View.VISIBLE);
            vh.tv_out_time.setText(entity.check_out_time);
            ImageLoadExtKt.loadImage(vh.iv_out, entity.check_out_img,
                    0, 0, false,
                    false, 0, null,
                    false, false);
            vh.fl_sign_out_box.setBackground(CommUtils.getDrawable(R.drawable.shape_sign_white_bg));
        } else {
            vh.iv_out.setVisibility(View.GONE);
            vh.tv_out.setVisibility(View.VISIBLE);
            vh.tv_out_time.setVisibility(View.GONE);
            vh.fl_sign_out_box.setBackground(CommUtils.getDrawable(R.drawable.shape_sign_green_bg));
        }

        if (entity.isExpend) {
            vh.iv_drop_down.setImageResource(R.drawable.home_up_icon);
            vh.ll_more_box.setVisibility(View.VISIBLE);
        } else {
            vh.iv_drop_down.setImageResource(R.drawable.home_down_icon);
            vh.ll_more_box.setVisibility(View.GONE);
        }

        vh.ll_name_box.setOnClickListener(v -> {
            entity.isExpend = !entity.isExpend;
            notifyDataSetChanged();
        });

        vh.fl_sign_in_box.setOnClickListener(v -> {
            if (mListener != null) mListener.onCheckIn(entity);
        });

        vh.fl_sign_out_box.setOnClickListener(v -> {
            if (mListener != null) mListener.onCheckOut(entity);
        });
    }


    private class IndexVH extends RecyclerView.ViewHolder {
        TextView tv;

        public IndexVH(View itemView) {
            super(itemView);
            tv = (TextView) itemView.findViewById(R.id.tv_index);
        }
    }

    private class ContentVH extends RecyclerView.ViewHolder {
        TextView tvName, tv_job_date, tv_start_time, tv_end_time, tv_in, tv_out, tv_in_time, tv_out_time;
        ImageView iv_in, iv_out, iv_drop_down;
        ViewGroup fl_sign_in_box, fl_sign_out_box, ll_more_box, ll_name_box;

        public ContentVH(View itemView) {
            super(itemView);
            tvName = (TextView) itemView.findViewById(R.id.tv_name);
            tv_job_date = (TextView) itemView.findViewById(R.id.tv_job_date);
            tv_start_time = (TextView) itemView.findViewById(R.id.tv_start_time);
            tv_end_time = (TextView) itemView.findViewById(R.id.tv_end_time);
            tv_in = (TextView) itemView.findViewById(R.id.tv_in);
            tv_out = (TextView) itemView.findViewById(R.id.tv_out);
            tv_in_time = (TextView) itemView.findViewById(R.id.tv_in_time);
            tv_out_time = (TextView) itemView.findViewById(R.id.tv_out_time);
            iv_out = (ImageView) itemView.findViewById(R.id.iv_out);
            iv_in = (ImageView) itemView.findViewById(R.id.iv_in);
            iv_drop_down = (ImageView) itemView.findViewById(R.id.iv_drop_down);
            fl_sign_in_box = itemView.findViewById(R.id.fl_sign_in_box);
            fl_sign_out_box = itemView.findViewById(R.id.fl_sign_out_box);
            ll_more_box = itemView.findViewById(R.id.ll_more_box);
            ll_name_box = itemView.findViewById(R.id.ll_name_box);
        }
    }


    private OnAttendanceListener mListener;

    public void setOnAttendanceListener(OnAttendanceListener listener) {
        mListener = listener;
    }

    public interface OnAttendanceListener {
        void onCheckIn(AttendanceBean entity);

        void onCheckOut(AttendanceBean entity);
    }
}
