package vn.hongyegroup.yybusiness.mvvm

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.viewModelScope
import com.android.basiclib.base.mvi.BaseEISViewModel
import com.android.basiclib.base.vm.BaseViewModel
import com.android.basiclib.bean.OkResult
import com.android.basiclib.engine.preferences.EasyDataStore
import com.android.basiclib.engine.toast.toast
import com.android.basiclib.utils.log.MyLogUtils
import com.android.cs_service.Constants
import dagger.hilt.android.lifecycle.HiltViewModel
import javax.inject.Inject


@HiltViewModel
class LoginViewModel @Inject constructor(
    private val repository: AppRepository,
    val savedState: SavedStateHandle
) : BaseViewModel() {


    /**
     * 执行登录
     */
    fun doLogin(code: String, password: String): LiveData<Boolean> {
        val liveData = MutableLiveData<Boolean>()
        launchOnUI {

            loadStartLoading()

            val loginResult = repository.requestLogin(code, password, "")

            if (loginResult is OkResult.Success) {
                //成功
                loadSuccess()

                val loginBean = loginResult.data
                EasyDataStore.put(Constants.KEY_TOKEN, loginBean.token)

                liveData.value = true
            } else {
                val message = (loginResult as OkResult.Error).exception.message
                loadError(message)
                toast(message)

                liveData.value = false
            }

        }
        return liveData
    }


}

