package vn.hongyegroup.yybusiness.mvvm

import android.annotation.SuppressLint
import androidx.fragment.app.FragmentActivity
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.SavedStateHandle
import com.android.basiclib.base.vm.BaseViewModel
import com.android.basiclib.bean.OkResult
import com.android.basiclib.engine.picker.pickDate
import com.android.basiclib.engine.preferences.EasyDataStore
import com.android.basiclib.engine.toast.toast
import com.android.basiclib.ext.timeStampFormat2Date
import com.android.basiclib.utils.CheckUtil
import com.android.cs_service.Constants
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import vn.hongyegroup.yybusiness.adapter.AttendanceAdapter
import vn.hongyegroup.yybusiness.entity.AttendanceBean
import vn.hongyegroup.yybusiness.entity.CheckInOut
import java.util.Date
import javax.inject.Inject


@HiltViewModel
class MainViewModel @Inject constructor(
    private val repository: AppRepository,
    val savedState: SavedStateHandle
) : BaseViewModel() {

    private var isNeedPlaceHolder = true
    var mKeyword: String? = null
    var mStartDateTimeStamp: Long = System.currentTimeMillis()
    var mEndDateTimeStamp: Long = System.currentTimeMillis()
    var mDatas = mutableListOf<AttendanceBean>()
    lateinit var mAdapter: AttendanceAdapter

    var pickTimeLD = MutableLiveData<Date>()

    /**
     * 获取考勤成员的列表数据
     */
    fun fetchAppliedList(): LiveData<Boolean> {
        val liveData = MutableLiveData<Boolean>()

        launchOnUI {

            //获取Token
            val token = withContext(Dispatchers.IO) {
                EasyDataStore.get(Constants.KEY_TOKEN, "")
            }

            //开始Loading
            if (isNeedPlaceHolder) loadStartLoading()

            val listResult = repository.fetchAppliedList(
                token,
                mKeyword,
                mStartDateTimeStamp.timeStampFormat2Date(),
                mEndDateTimeStamp.timeStampFormat2Date()
            )

            if (listResult is OkResult.Success) {
                //成功
                loadSuccess()
                handleData(listResult.data.list)
                liveData.value = true

            } else {
                val message = (listResult as OkResult.Error).exception.message
                loadError(message)
                toast(message)

                liveData.value = false
            }

        }

        return liveData
    }

    //处理数据-添加或刷新
    @SuppressLint("NotifyDataSetChanged")
    private fun handleData(list: List<AttendanceBean>?) {
        if (!CheckUtil.isEmpty(list)) {
            //有数据
            mDatas.clear()
            mDatas.addAll(list!!)
            mAdapter.setDatas(mDatas)

        } else {
            //展示无数据
            mDatas.clear()
            mAdapter.setDatas(mDatas)
            loadNoData()
        }

        isNeedPlaceHolder = false
    }


    /**
     * 执行退出登录
     */
    fun doLogout(): LiveData<Boolean> {
        val liveData = MutableLiveData<Boolean>()
        launchOnUI {

            loadStartProgress()

            //获取Token
            val token = withContext(Dispatchers.IO) {
                EasyDataStore.get(Constants.KEY_TOKEN, "")
            }

            val logoutResult = repository.requestLogout(token)

            if (logoutResult is OkResult.Success) {
                //成功
                loadHideProgress()

                //移除Token
                EasyDataStore.remove<String>(Constants.KEY_TOKEN)

                liveData.value = logoutResult.data

            } else {
                val message = (logoutResult as OkResult.Error).exception.message
               loadHideProgress()
                toast(message)

                liveData.value = false
            }

        }
        return liveData
    }

    /**
     * 筛选时间
     */
    fun pickerDate(isPickStart: Boolean, activity: FragmentActivity) {
        activity.pickDate(
            selectedDateTimeStamp = if (isPickStart) mStartDateTimeStamp else mEndDateTimeStamp,
            onConfirmAction = { date ->
                if (isPickStart) {
                    mStartDateTimeStamp = date.time
                } else {
                    mEndDateTimeStamp = date.time
                }

                pickTimeLD.value = date
            })
    }


    /**
     * 执行签到签出
     */
    fun requestCheckInOut(isCheckIn: Boolean, appliedId: String, path: String?): LiveData<CheckInOut> {
        val liveData = MutableLiveData<CheckInOut>()

        launchOnUI {

            //获取Token
            val token = withContext(Dispatchers.IO) {
                EasyDataStore.get(Constants.KEY_TOKEN, "")
            }

            //开始Loading
            loadStartProgress()

            val listResult = repository.requestCheckInOut(
                token,
                appliedId,
                isCheckIn,
                path
            )

            if (listResult is OkResult.Success) {
                //成功
                loadHideProgress()
                liveData.value = listResult.data

            } else {
                loadHideProgress()
                val message = (listResult as OkResult.Error).exception.message
                toast(message)
            }

        }

        return liveData
    }

}

