package vn.hongyegroup.yybusiness.ui

import android.os.Build
import android.os.Bundle
import android.text.method.HideReturnsTransformationMethod
import android.text.method.PasswordTransformationMethod
import android.view.WindowManager
import android.widget.CompoundButton
import androidx.lifecycle.Observer
import com.android.basiclib.base.activity.BaseVVDActivity
import com.android.basiclib.base.vm.EmptyViewModel
import com.android.basiclib.engine.toast.toast
import com.android.basiclib.ext.click
import com.android.basiclib.ext.gotoActivity
import com.android.basiclib.utils.CheckUtil
import com.android.basiclib.utils.StatusBarUtils
import dagger.hilt.android.AndroidEntryPoint
import vn.hongyegroup.yybusiness.R
import vn.hongyegroup.yybusiness.databinding.ActivityLoginBinding
import vn.hongyegroup.yybusiness.mvvm.LoginViewModel

/**
 * 登录页面
 */
@AndroidEntryPoint
class LoginActivity : BaseVVDActivity<LoginViewModel, ActivityLoginBinding>() {

    override fun init(savedInstanceState: Bundle?) {
        //沉浸式
        StatusBarUtils.immersive(this)

        //允许 window 的内容可以上移到刘海屏状态栏
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
            val lp = window.attributes
            lp.layoutInDisplayCutoutMode = WindowManager.LayoutParams.LAYOUT_IN_DISPLAY_CUTOUT_MODE_SHORT_EDGES
            window.setAttributes(lp)
        }

        initListener()

    }

    private fun initListener() {
        mBinding.cbPsd.setOnCheckedChangeListener { _: CompoundButton?, isChecked: Boolean ->
            if (isChecked) {
                mBinding.etPassword.transformationMethod = HideReturnsTransformationMethod.getInstance()
            } else {
                mBinding.etPassword.transformationMethod = PasswordTransformationMethod.getInstance()
            }

            // 确保光标移动到文本末尾
            mBinding.etPassword.setSelection(mBinding.etPassword.text.length)
        }


        mBinding.buttonLogin.click {
            val code = mBinding.etLoginCode.text.toString()
            val password = mBinding.etPassword.text.toString()
            checkParams(code, password)
        }
    }

    //校验参数并提交请求
    private fun checkParams(code: String, password: String) {
        if (CheckUtil.isEmpty(code)) {
            toast(R.string.please_enter_your_login_code)
            return
        }

        if (CheckUtil.isEmpty(code)) {
            toast(R.string.please_enter_your_password)
            return
        }

        mViewModel.doLogin(code, password).observe(this) {
            if (it != null && it) {
                gotoMainPage()
            }
        }
    }

    //去首页
    private fun gotoMainPage() {
        gotoActivity<MainActivity>()
        finish()
    }

}