package vn.hongyegroup.yybusiness.ui

import android.annotation.SuppressLint
import android.os.Build
import android.os.Bundle
import android.view.WindowManager
import androidx.lifecycle.lifecycleScope
import com.android.basiclib.base.activity.BaseVVDActivity
import com.android.basiclib.base.vm.EmptyViewModel
import com.android.basiclib.engine.preferences.EasyDataStore
import com.android.basiclib.ext.countDown
import com.android.basiclib.ext.gotoActivity
import com.android.basiclib.utils.CheckUtil
import com.android.basiclib.utils.StatusBarUtils
import com.android.basiclib.utils.log.MyLogUtils
import com.android.cs_service.Constants
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.ExperimentalCoroutinesApi
import kotlinx.coroutines.cancel
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import vn.hongyegroup.yybusiness.databinding.ActivitySplashBinding

@OptIn(ExperimentalCoroutinesApi::class)
@SuppressLint("CustomSplashScreen")
class SplashActivity : BaseVVDActivity<EmptyViewModel, ActivitySplashBinding>() {

    private var countDownScope: CoroutineScope? = null

    override fun init(savedInstanceState: Bundle?) {

        //沉浸式
        StatusBarUtils.immersive(this)

        //允许 window 的内容可以上移到刘海屏状态栏
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
            val lp = window.attributes
            lp.layoutInDisplayCutoutMode = WindowManager.LayoutParams.LAYOUT_IN_DISPLAY_CUTOUT_MODE_SHORT_EDGES
            window.setAttributes(lp)
        }

        countDown(time = 1,
            start = { scope ->
                countDownScope = scope
            },
            end = {
                //去登录页面还是去首页
                checkPage()

            },
            next = { time ->
                MyLogUtils.w("倒计时:$time")
            })

    }

    private fun checkPage() {

        lifecycleScope.launch {
            val token = withContext(Dispatchers.IO) {
                EasyDataStore.get(Constants.KEY_TOKEN, "")
            }

            if (!CheckUtil.isEmpty(token)) {
                gotoActivity<MainActivity>()
            } else {
                gotoActivity<LoginActivity>()
            }
            finish()
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        MyLogUtils.w("onDestroy")
        countDownScope?.cancel("Count Down Canceled By Destroy")
    }

}