package com.android.basiclib.engine.dialog.creator

import android.annotation.SuppressLint
import android.content.Context
import android.view.LayoutInflater
import androidx.viewbinding.ViewBinding
import com.android.basiclib.engine.dialog.callback.IPopupController
import com.lxj.xpopup.core.BasePopupView
import com.lxj.xpopup.impl.FullScreenPopupView

@SuppressLint("ViewConstructor")
private open class OpenFullScreenPopupView<VB : ViewBinding>(
    context: Context,

    private val viewBinding: ((LayoutInflater) -> VB)?,
    private val onCreateListener: ((VB, IPopupController) -> Unit)?,
    private val onDismissListener: (() -> Unit)?,
) : FullScreenPopupView(context) {

    //弹窗控制器-常见的功能
    private val closeControl = IPopupController { dismiss() }
    lateinit var mBinding: VB

    override fun addInnerContent() {
        if (viewBinding != null) {
            mBinding = viewBinding.invoke(LayoutInflater.from(context))
            fullPopupContainer.addView(mBinding.root)
        } else {
            super.addInnerContent()
        }
    }

    override fun onCreate() {
        super.onCreate()
        onCreateListener?.invoke(mBinding, closeControl)
    }

    override fun onDismiss() {
        super.onDismiss()
        onDismissListener?.invoke()
    }

}

class FullScreenPopupViewCreator<VB : ViewBinding>(
    private val context: Context,
    private val onCreateListener: ((VB, IPopupController) -> Unit)?,
    private val onDismissListener: (() -> Unit)?,
) : IPopupViewCreator<VB> {
    override fun create( viewBinding: ((LayoutInflater) -> VB)?): BasePopupView {
        return OpenFullScreenPopupView(
            context,
            viewBinding,
            onCreateListener, onDismissListener
        )
    }
}