package com.android.basiclib.engine.dialog.creator

import android.annotation.SuppressLint
import android.content.Context
import android.view.LayoutInflater
import androidx.viewbinding.ViewBinding
import com.android.basiclib.engine.dialog.callback.IPopupController
import com.lxj.xpopup.core.BasePopupView
import com.lxj.xpopup.impl.PartShadowPopupView

/**
 * 宽度自动占满一屏，高度自适应的弹窗，参照京东的筛选弹窗
 */
@SuppressLint("ViewConstructor")
private open class OpenPartPopupView<VB : ViewBinding>(
    context: Context,
    private val viewBinding: ((LayoutInflater) -> VB)?,
    private val height: Int,
    private val maxHeight: Int,
    private val onCreateListener: ((VB, IPopupController) -> Unit)?,
    private val onDismissListener: (() -> Unit)?,
) : PartShadowPopupView(context) {

    //弹窗控制器-常见的功能
    private val closeControl = IPopupController { dismiss() }

    lateinit var mBinding: VB

    override fun addInnerContent() {
        if (viewBinding != null) {
            mBinding = viewBinding.invoke(LayoutInflater.from(context))
            attachPopupContainer.addView(mBinding.root)
        } else {
            super.addInnerContent()
        }
    }

    override fun getPopupHeight(): Int {
        return if (height != 0) height else super.getPopupHeight()
    }

    override fun getMaxHeight(): Int {
        return if (maxHeight != 0) maxHeight else super.getMaxHeight()
    }

    override fun onCreate() {
        super.onCreate()
        onCreateListener?.invoke(mBinding, closeControl)
    }

    override fun onDismiss() {
        super.onDismiss()
        onDismissListener?.invoke()
    }

}

class PartPopupViewCreator<VB : ViewBinding>(
    private val context: Context,
    private val height: Int,
    private val maxHeight: Int,
    private val onCreateListener: ((VB, IPopupController) -> Unit)?,
    private val onDismissListener: (() -> Unit)?,
) : IPopupViewCreator<VB> {
    override fun create(viewBinding: ((LayoutInflater) -> VB)?): BasePopupView {
        return OpenPartPopupView(
            context,
            viewBinding,
            height,
            maxHeight,
            onCreateListener, onDismissListener
        )
    }
}