package com.android.basiclib.engine.dialog.creator

import android.content.Context
import androidx.viewbinding.ViewBinding
import com.android.basiclib.engine.dialog.PopupType
import com.android.basiclib.engine.dialog.callback.IPopupController

object PopupViewCreatorFactory {

    fun <VB : ViewBinding> getCreator(
        popupType: PopupType,
        context: Context,
        width: Int = 0,
        height: Int = 0,
        maxWidth: Int = 0,
        maxHeight: Int = 0,
        onCreateListener: ((VB, IPopupController) -> Unit)? = null,
        onDismissListener: (() -> Unit)? = null,
    ): IPopupViewCreator<VB> {
        return when (popupType) {
            PopupType.BOTTOM -> BottomPopupViewCreator(context, width, height, maxWidth, maxHeight, onCreateListener, onDismissListener)
            PopupType.KEYBOARD -> BottomPopupViewCreator(context, width, height, maxWidth, maxHeight, onCreateListener, onDismissListener)
            PopupType.CENTER -> CenterPopupViewCreator(context, width, height, maxWidth, maxHeight, onCreateListener, onDismissListener)
            PopupType.FULL_SCREEN -> FullScreenPopupViewCreator(context, onCreateListener, onDismissListener)
            PopupType.PART_SCREEN -> PartPopupViewCreator(context, height, maxHeight, onCreateListener, onDismissListener)
            PopupType.ATTACH -> AttachPopupViewCreator(context, width, height, maxWidth, maxHeight, onCreateListener, onDismissListener)
        }
    }
}