package com.android.basiclib.engine.image_grid

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.recyclerview.widget.RecyclerView
import com.android.basiclib.R
import com.android.basiclib.engine.image_load.loadImage
import com.android.basiclib.engine.preview.ImagePreviewEngine
import com.android.basiclib.ext.click

/**
 * 图片选择显示九宫格适配器
 */
class ImageGridAdapter(
    private var imageUrls: MutableList<String>,
    private val maxSelection: Int,
    private val addIconResId: Int,
    private val deleteIconResId: Int,
    private val placeholderResId: Int,
    private val addImageAction: () -> Unit,  //添加图片的回调
    private val delImageAction: (Int) -> Unit,  //删除图片的回调
) : RecyclerView.Adapter<ImageGridAdapter.ImageViewHolder>() {

    private var isSelectionMode: Boolean = true

    //手动设置展示模式，选择模式还是显示模式
    fun setSelectionMode(isSelectionMode: Boolean) {
        this.isSelectionMode = isSelectionMode
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ImageViewHolder {
        val view = LayoutInflater.from(parent.context).inflate(R.layout.item_image_grid, parent, false)
        return ImageViewHolder(view)
    }

    override fun onBindViewHolder(holder: ImageViewHolder, position: Int) {
        if (isSelectionMode) {
            if (imageUrls.size > 0 && position < imageUrls.size) {
                val item = imageUrls[position]
                //绑定数据,设置图片
                holder.ivImage.loadImage(item, placeholderResId)
                holder.ivDelete.setImageResource(deleteIconResId)

                //设置状态
                holder.ivDelete.visibility = View.VISIBLE

                //设置点击事件
                holder.ivDelete.click {
                    delImageAction(position)
                }

                //调用图片预览引擎预览多图
                holder.ivImage.click {
                    ImagePreviewEngine.multipleImagePreview(it.context, holder.ivImage, imageUrls, position,
                        updateAction = { controller, index ->
                            val rv = holder.itemView.parent as RecyclerView
                            controller.updateSrcView(rv.getChildAt(index).findViewById(R.id.iv_image))
                        })
                }

            } else {
                // 最后一个位置显示 "Add" 图片
                holder.ivImage.setImageResource(addIconResId)
                holder.ivDelete.visibility = View.GONE
                holder.ivImage.click {
                    //添加图片的选择
                    addImageAction()
                }
            }

        } else {
            //纯展示模式，没有添加和删除的功能
            val item = imageUrls[position]
            //绑定数据,设置图片
            holder.ivImage.loadImage(item, placeholderResId)
            holder.ivDelete.setImageResource(deleteIconResId)
            holder.ivDelete.visibility = View.GONE

            //调用图片预览引擎预览多图
            holder.ivImage.click {
                ImagePreviewEngine.multipleImagePreview(it.context, holder.ivImage, imageUrls, position,
                    updateAction = { controller, index ->
                        val rv = holder.itemView.parent as RecyclerView
                        controller.updateSrcView(rv.getChildAt(index).findViewById(R.id.iv_image))
                    })
            }

        }

    }

    override fun getItemCount(): Int {
        return if (!isSelectionMode) {
            imageUrls.size
        } else {
            if (imageUrls.size >= maxSelection) {
                imageUrls.size
            } else {
                imageUrls.size + 1
            }
        }
    }

    inner class ImageViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val ivImage: ImageView = itemView.findViewById(R.id.iv_image)
        val ivDelete: ImageView = itemView.findViewById(R.id.iv_delete)
    }
}