package com.android.basiclib.engine.image_grid

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.util.AttributeSet
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.android.basiclib.R
import com.android.basiclib.engine.image_select.dialog.ImageSelectDialogEngine
import com.luck.picture.lib.decoration.GridSpacingItemDecoration

/**
 * 图片选择显示九宫格，根据模式可以选择图片，可以显示图片，常用于提交多图片场景，如发布朋友圈，提交意见反馈图片等
 */
class ImageGridView(context: Context, attrs: AttributeSet) : RecyclerView(context, attrs) {

    private var maxSelection: Int = 0
    private var addIconResId: Int = 0
    private var deleteIconResId: Int = 0
    private var placeholderResId: Int = 0
    private var isSelectionMode: Boolean = true
    private var gridSpacing: Int = 0
    private val images: MutableList<String> = mutableListOf()
    private lateinit var imageAdapter: ImageGridAdapter

    init {
        val a = context.obtainStyledAttributes(attrs, R.styleable.ImageGridView)
        maxSelection = a.getInt(R.styleable.ImageGridView_maxSelection, 9)
        addIconResId = a.getResourceId(R.styleable.ImageGridView_addIcon, R.drawable.img_picture_add)
        deleteIconResId = a.getResourceId(R.styleable.ImageGridView_deleteIcon, R.drawable.img_picture_delete)
        placeholderResId = a.getResourceId(R.styleable.ImageGridView_placeholder, com.luck.picture.lib.R.drawable.ps_image_placeholder)
        isSelectionMode = a.getBoolean(R.styleable.ImageGridView_isSelectionMode, true)
        gridSpacing = a.getDimensionPixelSize(R.styleable.ImageGridView_gridSpacing, 10)
        a.recycle()
    }

    //手动设置展示模式，选择模式还是显示模式
    fun setSelectionMode(isSelectionMode: Boolean) {
        this.isSelectionMode = isSelectionMode
    }

    fun setMaxSelection(maxSelection: Int) {
        this.maxSelection = maxSelection
    }

    fun setAddIconResId(addIconResId: Int) {
        this.addIconResId = addIconResId
    }

    fun setDeleteIconResId(deleteIconResId: Int) {
        this.deleteIconResId = deleteIconResId
    }

    fun setPlaceholderResId(placeholderResId: Int) {
        this.placeholderResId = placeholderResId
    }

    fun setGridSpacing(gridSpacing: Int) {
        this.gridSpacing = gridSpacing
    }

    //获取当前已经选取的图片集合
    fun getImageList(): List<String> = images

    /**
     * 设置类型与数据
     */
    @SuppressLint("NotifyDataSetChanged")
    fun setup(isSelectionMode: Boolean, list: List<String>? = null) {

        // 设置布局管理器和间距
        val layoutManager = GridLayoutManager(context, 3)
        layoutManager.spanSizeLookup = object : GridLayoutManager.SpanSizeLookup() {
            override fun getSpanSize(position: Int): Int {
                return 1
            }
        }
        layoutManager.spanCount = 3
        this.layoutManager = layoutManager

        this.addItemDecoration(GridSpacingItemDecoration(3, gridSpacing, true))

        if (!list.isNullOrEmpty()) {
            images.clear()
            images.addAll(list)
        }

        imageAdapter = ImageGridAdapter(images, maxSelection, addIconResId, deleteIconResId, placeholderResId,
            addImageAction = {
                addImagePick()
            },
            delImageAction = {
                deleteImageByPosition(it)
            }
        )

        this.isSelectionMode = isSelectionMode
        imageAdapter.setSelectionMode(this.isSelectionMode)

        this.adapter = imageAdapter

    }

    //删除图片
    @SuppressLint("NotifyDataSetChanged")
    private fun deleteImageByPosition(position: Int) {
        images.removeAt(position)
        imageAdapter.notifyDataSetChanged()
    }

    //添加图片
    @SuppressLint("NotifyDataSetChanged")
    private fun addImagePick() {
        val activity = context as? Activity
        activity?.let {
            ImageSelectDialogEngine.imageSelect(it, maxNumFromAlbum = maxSelection - images.size) {
                images.addAll(it)
                imageAdapter.notifyDataSetChanged()
            }
        }

    }
}
