package com.android.basiclib.engine.image_select

import android.content.Context
import android.net.Uri
import com.android.basiclib.utils.log.MyLogUtils
import com.luck.picture.lib.config.PictureMimeType
import com.luck.picture.lib.engine.CompressFileEngine
import com.luck.picture.lib.interfaces.OnKeyValueResultCallbackListener
import com.luck.picture.lib.utils.DateUtils
import top.zibin.luban.Luban
import top.zibin.luban.OnNewCompressListener
import java.io.File

/**
 * 图片压缩引擎，可用于 ImageSelectEngine 中设置，实现CompressFileEngine可以在 ImageSelectEngine 中自动配置相机和相册后的压缩
 *
 * 也可以单独使用，设置本地图片的Uri和目标Path的Uri，即可单独实现裁剪
 */
class ImageFileCompressEngine : CompressFileEngine {
    /**
     * 自定义方法，可以自己手动压缩图片，传入路径即可
     */
    fun onStartSanboxCompress(context: Context?, paths: ArrayList<String>?, call: OnKeyValueResultCallbackListener?) {

        Luban.with(context).load(paths).ignoreBy(100) //最大100K
            .setRenameListener { filePath ->
                val indexOf = filePath.lastIndexOf(".")
                val postfix = if (indexOf != -1) filePath.substring(indexOf) else ".jpg"
                DateUtils.getCreateFileName("CMP_") + postfix
            }.filter { path -> //是带图片后缀的图片，并且不含有Http的，不是gif图片
                if (PictureMimeType.isUrlHasImage(path) &&
                    !PictureMimeType.isHasHttp(path) &&
                    !PictureMimeType.isUrlHasGif(path)
                ) {
                    MyLogUtils.w("符合条件可以使用Luban压缩")
                    true
                } else {
                    MyLogUtils.w("不符合条件不能使用Luban压缩")
                    false
                }
            }.setCompressListener(object : OnNewCompressListener {
                override fun onStart() {
                    MyLogUtils.w("开始Luban压缩")
                }

                override fun onSuccess(source: String, compressFile: File) {
                    MyLogUtils.w("Luban压缩成功 source:" + source + " compressFile:" + compressFile.absolutePath)
                    call?.onCallback(source, compressFile.absolutePath)
                }

                override fun onError(source: String, e: Throwable) {
                    MyLogUtils.w("Luban压缩失败 source：$source")
                    call?.onCallback(source, null)
                }
            }).launch()
    }

    /**
     * 重写的接口方法，专用于 Image Select 中的图片自动压缩
     */
    override fun onStartCompress(context: Context, source: ArrayList<Uri>, call: OnKeyValueResultCallbackListener) {
        Luban.with(context).load(source).ignoreBy(100) //最大100K
            .setRenameListener { filePath ->
                val indexOf = filePath.lastIndexOf(".")
                val postfix = if (indexOf != -1) filePath.substring(indexOf) else ".jpg"
                DateUtils.getCreateFileName("CMP_") + postfix
            }.filter { path -> //是带图片后缀的图片，并且不含有Http的，不是gif图片
                if (PictureMimeType.isUrlHasImage(path) &&
                    !PictureMimeType.isHasHttp(path) &&
                    !PictureMimeType.isUrlHasGif(path)
                ) {
                    MyLogUtils.w("符合条件可以使用Luban压缩")
                    true
                } else {
                    MyLogUtils.w("不符合条件不能使用Luban压缩")
                    false
                }
            }.setCompressListener(object : OnNewCompressListener {
                override fun onStart() {
                    MyLogUtils.w("开始Luban压缩")
                }

                override fun onSuccess(source: String, compressFile: File) {
                    MyLogUtils.w("Luban压缩成功 source:" + source + " compressFile:" + compressFile.absolutePath)
                    call.onCallback(source, compressFile.absolutePath)

                }

                override fun onError(source: String, e: Throwable) {
                    MyLogUtils.w("Luban压缩失败 source：$source")
                    call.onCallback(source, null)

                }
            }).launch()
    }
}