package com.android.basiclib.engine.picker

import android.view.View
import androidx.fragment.app.FragmentActivity
import com.lxj.xpopup.XPopup
import com.lxj.xpopupext.listener.TimePickerListener
import com.lxj.xpopupext.popup.TimePickerPopup
import java.util.Calendar
import java.util.Date


/**
 * 展示时间的筛选Picker
 */
fun FragmentActivity.pickDate(
    selectedDateTimeStamp: Long? = null,
    startRange: Int = -10, //时间筛选范围，前10年
    endRange: Int = 10,  //时间筛选范围，后10年
    onChangeAction: ((date: Date) -> Unit)? = null,  //改变
    onConfirmAction: ((date: Date) -> Unit)? = null,  //确认
    onCancelAction: (() -> Unit)? = null,     //取消回调
) {

    val popup = TimePickerPopup(this)
        .setItemTextSize(16)
        .setShowLabel(true)
        .setCyclic(false)


    //设置默认选中日期
    if (selectedDateTimeStamp != null) {
        popup.setDefaultDate(Calendar.getInstance().apply {
            timeInMillis = selectedDateTimeStamp
        })
    }

    //设置时间筛选的范围
    popup.setDateRange(Calendar.getInstance().apply {
        add(Calendar.YEAR, startRange)
    }, Calendar.getInstance().apply {
        add(Calendar.YEAR, endRange)
    })

    popup.setTimePickerListener(object : TimePickerListener {
        override fun onTimeChanged(date: Date) {
            //时间改变
            onChangeAction?.invoke(date)
        }

        override fun onTimeConfirm(date: Date, view: View?) {
            //点击确认时间
            onConfirmAction?.invoke(date)
        }

        override fun onCancel() {
            //取消
            onCancelAction?.invoke()
        }
    })

    XPopup.Builder(this)
        .asCustom(popup)
        .show()

}