package com.android.basiclib.engine.preview

import android.content.Context
import android.graphics.Color
import android.widget.ImageView
import com.lxj.xpopup.XPopup
import com.lxj.xpopup.util.SmartGlideImageLoader

/**
 * 图片的预览引擎类，目前基于XPopup实现
 */
object ImagePreviewEngine {

    /**
     * 单图片的预览
     */
    fun singleImagePreview(
        context: Context,
        img: ImageView,
        imageUrl: String,
        placeholderColor: Int = -1,
        isInfinite: Boolean = false,
        placeholderStroke: Int = -1,
        roundRadius: Int = 0,
        isShowSaveBtn: Boolean = false,
        bgColor: Int = Color.BLACK,
        longPressListener: ImagePreviewLongPressListener? = null
    ) {

        val adapter = longPressListener?.let {
            ImagePreviewLongPressListenerAdapter(it)
        }

        XPopup.Builder(context)
            .asImageViewer(
                img,
                imageUrl,
                isInfinite,
                placeholderColor,
                placeholderStroke,
                roundRadius,
                isShowSaveBtn,
                bgColor,
                SmartGlideImageLoader(),
                adapter
            )
            .show()
    }

    /**
     * 多图的选择
     */
    fun multipleImagePreview(
        context: Context,
        img: ImageView,
        list: List<Any>,
        position: Int = 0,
        placeholderRes: Int = 0,
        isInfinite: Boolean = false,
        placeholderStroke: Int = -1,
        isShowSaveBtn: Boolean = false,
        roundRadius: Int = 0,
        bgColor: Int = Color.BLACK,
        updateAction: ImagePreviewUpdateListener? = null,
        longPressListener: ImagePreviewLongPressListener? = null
    ) {

        val updateAdapter = updateAction?.let {
            ImagePreviewUpdateListenerAdapter(it)
        }

        val longPressAdapter = longPressListener?.let {
            ImagePreviewLongPressListenerAdapter(it)
        }


        XPopup.Builder(context).asImageViewer(
            img,
            position,
            list,
            isInfinite,
            placeholderRes != 0,
            placeholderRes,
            placeholderStroke,
            roundRadius,
            isShowSaveBtn,
            bgColor,
            updateAdapter,
            SmartGlideImageLoader(),
            longPressAdapter
        ).show()
    }


}